#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

import ConfigParser

from spider163 import version


PATH = os.environ.get("HOME") + "/spider163"
if os.environ.get("SPIDER163_PATH") is not None:
    PATH = os.environ.get("SPIDER163_PATH")

if not os.path.exists(PATH):
    os.makedirs(PATH)
cf = ConfigParser.ConfigParser()
if not os.path.exists(PATH + "/spider163.conf"):
    print("请在默认路径 " + PATH + " 下增加配置文件 spider163.conf 格式参照官方")
    cf.read("{}/template/spider163.conf".format(version.root_path))
else:
    cf.read(PATH + "/spider163.conf")


def get_path():
    return PATH


def get_db():
    try:
        return cf.get("core", "db")
    except Exception as e:
        print("配置文件存在问题，请在 {}/spider163.conf 中配置db=xxx选项".format(PATH))
        print("错误详情： {}".format(e))
        raise e


def format_db():
    """db=mysql://root:password@127.0.0.1/spider?charset=utf8mb4"""
    link = get_db()
    r = re.search("mysql:\/\/([^:]+):([^@]+)@((?:[0-9]{1,3}\.){3}[0-9]{1,3})/([^\?]+)\?charset=utf8mb4", link)
    if r is None:
        return r
    else:
        return {
            "link": r.group(0),
            "user": r.group(1),
            "password": r.group(2),
            "ip": r.group(3),
            "database": r.group(4)
        }


def get_mysql():
    link = get_db()
    db = re.search('(?<=/)[^/]+(?=\?)', link).group(0)
    uri = re.search('.*(?=/)', link).group(0)
    return {"db": db, "uri": uri}


def get_port():
    try:
        return int(cf.get("core", "port"))
    except Exception as e:
        print("配置文件存在问题，请在 {}/spider163.conf 中配置port=xxx选项".format(PATH))
        print("错误详情： {}".format(e))
        raise e







#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from spider163.utils import config
from logbook import FileHandler, Logger
from terminaltables import AsciiTable
from colorama import Fore
from colorama import init

path = config.get_path()
log_handler = FileHandler(filename=path + '/spider163.log')
log_handler.push_application()
log = Logger("")

init(autoreset=True)


def Log(msg):
    log.warn(msg)


def Table(tb):
    print(AsciiTable(tb).table)


def Blue(msg):
    return Fore.BLUE + msg


def green(msg):
    return Fore.GREEN + msg


def red(msg):
    return Fore.RED + msg


def print_err(msg):
    print(Fore.RED + msg)


def print_warn(msg):
    print(Fore.YELLOW + msg)


def print_info(msg):
    print(Fore.BLUE + msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
from terminaltables import AsciiTable

from spider163.utils import pylog
import settings as uapi

offset = 0
limit = 20
type = {1: "歌曲", 10: "专辑", 100: "歌手", 1000: "歌单"}


def searchSong(key):
    url = uapi.search_api
    data = {'s': key, 'offset': 0, 'limit': 20, 'type': "1"}
    req = requests.post(url, headers=uapi.header, data=data, timeout=10)
    if req.json()["result"]['songCount'] == 0:
        pylog.log.warn("关键词 {} 没有可搜索歌曲".format(key))
        return
    songs = req.json()["result"]['songs']
    song_table = AsciiTable([["ID", "歌曲", "专辑", "演唱"]])
    for item in songs:
        id = item['id']
        name = (item['name']).encode("utf-8")
        album = (item['album']['name']).encode("utf-8")
        artist = []
        for a in item['artists']:
            artist.append(a['name'].encode("utf-8"))
        song_table.table_data.append([str(id), name, album, ",".join(artist)])
    print(pylog.Blue("与 \"{}\" 有关的歌曲".format(key)))
    print(song_table.table)


def searchAlbum(key):
    url = uapi.search_api
    data = {'s': key, 'offset': 0, 'limit': 20, 'type': "10"}
    req = requests.post(url, headers=uapi.header, data=data, timeout=10)
    if req.json()["result"]['albumCount'] == 0:
        pylog.log.warn("关键词 {} 没有可搜索专辑".format(key))
        return
    albums = req.json()["result"]['albums']
    song_table = AsciiTable([["ID", "专辑", "演唱","发行方"]])
    for item in albums:
        id = item['id']
        name = item['name'].encode("utf-8")
        company = ""
        if item['company'] !=  None:
            company = item['company'].encode("utf-8")
        artist = []
        for a in item['artists']:
            artist.append(a['name'].encode("utf-8"))
        song_table.table_data.append([str(id), name, ",".join(artist), company])
    print(pylog.Blue("与 \"{}\" 有关的专辑".format(key)))
    print(song_table.table)


def searchSinger(key):
    url = uapi.search_api
    data = {'s': key, 'offset': 0, 'limit': 10, 'type': "100"}
    req = requests.post(url, headers=uapi.header, data=data, timeout=10)
    if req.json()["result"]['artistCount'] == 0:
        pylog.log.warn("关键词 {} 没有可搜索艺术家".format(key))
        return
    artists = req.json()["result"]['artists']
    song_table = AsciiTable([["ID", "姓名", "专辑数量", "MV数量"]])
    for item in artists:
        id = str(item['id'])
        name = item['name'].encode("utf-8")
        acount = str(item['albumSize'])
        mcount = str(item['mvSize'])
        song_table.table_data.append([id, name, acount, mcount])
    print(pylog.Blue("与 \"{}\" 有关的歌手".format(key)))
    print(song_table.table)


def searchPlaylist(key):
    url = uapi.search_api
    data = {'s': key, 'offset': 0, 'limit': 5, 'type': "1000"}
    req = requests.post(url, headers=uapi.header, data=data, timeout=10)
    if req.json()["result"]['playlistCount'] == 0:
        pylog.log.warn("关键词 {} 没有可搜索歌单".format(key))
        return
    playlists = req.json()["result"]['playlists']
    song_table = AsciiTable([["ID", "歌单", "维护者", "播放数量", "收藏数量"]])
    for item in playlists:
        id = str(item['id'])
        name = item['name'].encode("utf-8")
        creator = item['creator']['nickname'].encode("utf-8")
        pcount = str(item['playCount'])
        bcount = str(item['bookCount'])
        song_table.table_data.append([id, name, creator, pcount, bcount])
    print(pylog.Blue("与 \"{}\" 有关的歌单".format(key)))
    print(song_table.table)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import json

from bs4 import BeautifulSoup

import settings as uapi
from spider163 import settings
from spider163.utils import pysql
from spider163.utils import pylog


class Lyric:

    def __init__(self):
        self.__headers = uapi.header
        self.session = settings.Session()

    def view_lyric(self, song_id):
        url = uapi.lyric_url.format(str(song_id))
        s = requests.session()
        try:
            s = BeautifulSoup(s.get(url, headers=self.__headers).content, "html.parser")
            lrc = json.loads(s.text)['lrc']['lyric']
            if pysql.single("lyric163", "song_id", song_id):
                self.session.add(pysql.Lyric163(song_id=song_id, txt=lrc))
                self.session.query(pysql.Music163).filter(pysql.Music163.song_id == song_id).update({"has_lyric": "Y"})
                self.session.commit()
        except Exception as e:
            self.session.query(pysql.Music163).filter(pysql.Music163.song_id == song_id).update({"has_lyric": "E"})
            self.session.commit()
            pylog.log.error("抓取歌词出现问题：{} 歌曲ID：{}".format(e, song_id))
            # raise

    def get_lyric(self, song_id):
        self.view_lyric(song_id)
        lrc = self.session.query(pysql.Lyric163).filter(pysql.Lyric163.song_id == song_id)
        print(lrc[0].txt)

    def view_lyrics(self, count):
        song = []
        for i in range(count/10):
            ms = self.session.query(pysql.Music163).filter(pysql.Music163.has_lyric == "N").limit(10)
            for m in ms:
                print("正在抓取歌词 ID {} 歌曲 {}".format(m.song_id, pylog.Blue(m.song_name.encode("utf-8"))))
                self.view_lyric(m.song_id)
                song.append({"name": m.song_name,"author": m.author,"comment": m.comment})
        ms = self.session.query(pysql.Music163).filter(pysql.Music163.has_lyric == "N").limit(count%10)
        for m in ms:
            print("正在抓取歌词 ID {} 歌曲 {}".format(m.song_id, pylog.Blue(m.song_name.encode("utf-8"))))
            self.view_lyric(m.song_id)
            song.append({"name": m.song_name, "author": m.author, "comment": m.comment})
        return song



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from flask import Flask, request, json, jsonify
from flask import render_template, make_response

from spider163.spider import playlist
from spider163.spider import music
from spider163.spider import lyric
from spider163.spider import comment
from spider163.utils import pysql

app = Flask(__name__, static_path='/static')


@app.route("/")
def index():
    return render_template('index.html')


@app.route("/spider")
def spider(type=None):
    return render_template('spider.html')


@app.route("/spider/getPlaylist", methods=['POST'])
def get_playlist():
    pl = playlist.Playlist()
    title = pl.view_capture(int(request.form['gdPage']),request.form["gdType"].encode("utf-8"))
    return jsonify({"type": request.form["gdType"],"title": title})


@app.route("/spider/getMusic", methods=['POST'])
def get_music():
    mu = music.Music()
    data = mu.views_capture(request.form["gdSource"].encode("utf-8"))
    return jsonify({"type": request.form["gdSource"],"data": data})


@app.route("/spider/getLyric", methods=["POST"])
def get_lyric():
    ly = lyric.Lyric()
    data = ly.view_lyrics(int(request.form["gqCount"]))
    return jsonify({"count": request.form["gqCount"],"data": data})


@app.route("/spider/getComment", methods=["POST"])
def get_comment():
    cm = comment.Comment()
    data = cm.auto_view(int(request.form["gqCount"]))
    return jsonify({"count": request.form["gqCount"],"data": data})


@app.route("/stat")
def statistics():
    return render_template('stat.html')


@app.route("/stat/playlist")
def stat_playlist():
    return jsonify(pysql.stat_playlist())


@app.route("/stat/music")
def stat_music():
    return jsonify(pysql.stat_music())


@app.route("/stat/dataCount")
def stat_data():
    return jsonify(pysql.stat_data())


@app.route("/scan")
def scan():
    return render_template('scan.html')


@app.route("/scan/data")
def scan_data():
    comment = pysql.random_data()
    if len(comment) > 0:
        res = {"msg":"ok","num":len(comment),"comment":comment}
        return jsonify(res)
    else:
        return jsonify({"msg":"ok","num":len(comment),"comment":[]})


@app.route("/bussiness")
def bussiness():
    return render_template('bussiness.html')







#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
from bs4 import BeautifulSoup
from terminaltables import AsciiTable

from spider163.utils import pysql
from spider163.utils import pylog
from spider163 import settings
import settings as uapi


class Playlist:
    __play_url = None
    __headers = None

    def __init__(self):
        self.__headers = uapi.header
        self.__play_url = uapi.play_url
        self.session = settings.Session()

    def get_classify(self):
        table = [["类别", "风格列表"]]
        for k, v in uapi.classify.items():
            c = 0
            lst = ""
            for v in v:
                c = c + 1
                if c % 5 == 0:
                    lst = lst + v + "\n"
                else:
                    lst = lst + v + ","
            table.append([k, lst])
        print(AsciiTable(table).table)

    def view_capture(self, page, type="全部"):
        s = requests.session()
        play_url = self.__play_url.format(type, page * 35)
        titles = []
        try:
            acmsk = {'class': 'msk'}
            scnb = {'class': 'nb'}
            dcu = {'class': 'u-cover u-cover-1'}
            ucm = {'class': 'm-cvrlst f-cb'}
            s = BeautifulSoup(s.get(play_url, headers=self.__headers).content, "html.parser")
            lst = s.find('ul', ucm)
            for play in lst.find_all('div', dcu):
                title = play.find('a', acmsk)['title'].encode('utf-8')
                link = play.find('a', acmsk)['href'].encode('utf-8').replace("/playlist?id=", "")
                cnt = play.find('span', scnb).text.encode('utf-8').replace('万', '0000')
                if pysql.single("playlist163","link",link) is True:
                    pl = pysql.Playlist163(title=title, link=link, cnt=int(cnt), dsc="曲风：{}".format(type))
                    self.session.add(pl)
                    self.session.commit()
                    titles.append(title)
            return titles
        except Exception as e:
            pylog.log.error("抓取歌单出现问题：{} 歌单类型：{} 页码：{}".format(e, type, page))
            raise



#!/usr/bin/env python
# -*- coding: utf-8 -*-

header = {
            'Referer': 'http://music.163.com/',
            'Host': 'music.163.com',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.95 Safari/537.36',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'
        }
comment_text = {
        'username': '13393376853',
        'password': 'wangyidafahao',
        'rememberLogin': 'true'
        }
comment_module = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
pubKey = '010001'
secKey = 16 * 'F'
comment_url = "http://music.163.com/weapi/v1/resource/comments/R_SO_4_{}/?csrf_token="
lyric_url = "http://music.163.com/api/song/lyric?os=pc&id={}&lv=-1&kv=-1&tv=-1"
play_url = "http://music.163.com/discover/playlist/?order=hot&cat={}&limit=35&offset={}"
music_url = "http://music.163.com/api/v3/playlist/detail?id="

playlist_api = "http://music.163.com/api/playlist/detail?id={}&upd"

music_api = "http://music.163.com/api/song/detail/?id={}&ids=[{}]"

search_api = "http://music.163.com/api/search/pc"

classify = {
    "语种":["华语", "欧美", "日语","韩语", "粤语", "小语种", ],
    "风格":["流行", "摇滚", "民谣", "电子", "舞曲", "说唱", "轻音乐", "爵士", "乡村", "R&B/Soul", "古典", "民族", "英伦", "金属", "朋克", "蓝调", "雷鬼", "世界音乐", "拉丁", "另类/独立", "New Age", "古风", "后摇", "Bossa Nova"],
    "场景":["清晨", "夜晚", "学习", "工作", "午休", "下午茶", "地铁", "驾车", "运动", "旅行", "散步", "酒吧"],
    "情感":["怀旧", "清新", "浪漫", "性感", "伤感", "治愈", "放松", "孤独", "感动", "兴奋", "快乐", "安静", "思念"],
    "主题":["影视原声", "ACG", "校园", "游戏", "70后", "80后", "90后", "网络歌曲", "KTV", "经典", "翻唱", "吉他", "钢琴", "器乐", "儿童", "榜单", "00后"]
}



# -*- coding: utf-8 -*-

from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker
from spider163.utils import config
from sqlalchemy_utils.functions import database_exists

from spider163.utils import pylog


def configure_orm():
    global engine
    global Session
    engine_args = {}
    try:
        if database_exists(config.get_db()) is False:
            create_engine(config.get_mysql()['uri'], echo=False).execute("create database IF NOT EXISTS  {} DEFAULT CHARACTER SET utf8".format(config.get_mysql()['db']))
        engine = create_engine(config.get_db(), **engine_args)
        Session = scoped_session(
            sessionmaker(autocommit=False, autoflush=False, bind=engine))
    except Exception as e:
        pylog.print_err("初始化数据库出现问题： {}".format(e))







#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
import json
from bs4 import BeautifulSoup

import settings as uapi
from spider163 import settings
from spider163.utils import pysql
from spider163.utils import pylog
from terminaltables import AsciiTable


class Music:
    
    def __init__(self):
        self.__headers = uapi.header
        self.__url = uapi.music_url
        self.session = settings.Session()

    def views_capture(self,source=None):
        playlist = {}
        if source is None:
            urls = self.session.query(pysql.Playlist163).filter(pysql.Playlist163.over == 'N').limit(10)
        else:
            if source.startswith("曲风：") is False:
                source = "曲风：" + source
            urls = self.session.query(pysql.Playlist163).filter(pysql.Playlist163.over == 'N',pysql.Playlist163.dsc==source).limit(1)
        for url in urls:
            print("正在抓取歌单《{}》的歌曲……".format(url.title.encode("utf-8")))
            songs = self.view_capture(url.link)
            playlist[url.title.encode("utf-8")] = songs
        for url in urls:
            self.session.query(pysql.Playlist163).filter(pysql.Playlist163.link == url.link).update({'over': 'Y'})
            self.session.commit()
        return playlist

    def view_capture(self, link):
        self.session.query(pysql.Playlist163).filter(pysql.Playlist163.link == link).update({'over': 'Y'})
        url = self.__url + str(link)
        s = requests.session()
        songs = []
        try:
            s = BeautifulSoup(s.get(url, headers=self.__headers).content, "html.parser")
            musics = json.loads(s.text)['playlist']['tracks']
            exist = 0
            for music in musics:
                name = music['name'].encode('utf-8')
                author = music['ar'][0]['name'].encode('utf-8')
                if pysql.single("music163", "song_id", (music['id'])) is True:
                    self.session.add(pysql.Music163(song_id=music['id'],song_name=name,author=author))
                    self.session.commit()
                    exist = exist + 1
                    songs.append({"name": name,"author": author})
                else:
                    pylog.log.info('{} : {} {}'.format("重复抓取歌曲", name, "取消持久化"))
            print("歌单包含歌曲 {} 首,数据库 merge 歌曲 {} 首 \r\n".format(len(musics), exist))
            return songs
        except Exception as e:
            pylog.log.error("抓取歌单页面存在问题：{} 歌单ID：{}".format(e, url))
            raise

    def get_playlist(self, playlist_id):
        self.view_capture(int(playlist_id))
        url = uapi.playlist_api.format(playlist_id)
        s = requests.session()
        s = BeautifulSoup(s.get(url, headers=self.__headers).content, "html.parser")
        playlist = json.loads(s.text)['result']

        print("《" + playlist['name'].encode('utf-8') + "》")
        author = playlist['creator']['nickname'].encode('utf-8')
        pc = str(playlist['playCount'])
        sc = str(playlist['subscribedCount'])
        rc = str(playlist['shareCount'])
        cc = str(playlist['commentCount'])
        print("维护者：{}  播放：{} 关注：{} 分享：{} 评论：{}".format(author, pc, sc, rc, cc))
        print("描述：{}".format(playlist['description'].encode('utf-8')))
        print("标签：{}".format(",".join(playlist['tags']).encode("utf-8")))

        tb = [["ID", "歌曲名字", "艺术家", "唱片"]]
        for music in playlist['tracks']:
            artists = []
            for s in music['ar']:
                artists.append(s['name'])
            ms = music['name'].encode("utf-8")
            ar = ",".join(artists).encode("utf-8")
            ab = music['album']['name'].encode("utf-8")
            id = music['id']
            tb.append([id, ms, ar, ab])
        print(AsciiTable(tb).table)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import os
import base64
import json

from Crypto.Cipher import AES
from bs4 import BeautifulSoup
from terminaltables import AsciiTable


import settings as uapi
from spider163 import settings
from spider163.utils import pysql
from spider163.utils import pylog


class Comment:

    def __init__(self):
        self.__headers = uapi.header
        self.session = settings.Session()
        text = uapi.comment_text
        modulus = uapi.comment_module
        pubKey = uapi.pubKey
        secKey = uapi.secKey
        self.__encSecKey = self.rsaEncrypt(secKey, pubKey, modulus)

    def createParams(self, page=1):
        if page == 1:
            text = '{rid:"", offset:"0", total:"true", limit:"20", csrf_token:""}'
        else:
            offset = str((page-1)*20)
            text = '{rid:"", offset:"%s", total:"%s", limit:"20", csrf_token:""}' %(offset,'false')
        nonce = '0CoJUm6Qyw8W8jud'
        nonce2 = 16 * 'F'
        encText = self.aesEncrypt(self.aesEncrypt(text,nonce),nonce2)
        return encText

    def aesEncrypt(self, text, secKey):
        pad = 16 - len(text) % 16
        text = text + pad * chr(pad)
        encryptor = AES.new(secKey, 2, '0102030405060708')
        ciphertext = encryptor.encrypt(text)
        ciphertext = base64.b64encode(ciphertext)
        return ciphertext

    def rsaEncrypt(self, text, pubKey, modulus):
        text = text[::-1]
        rs = int(text.encode('hex'), 16)**int(pubKey, 16) % int(modulus, 16)
        return format(rs, 'x').zfill(256)

    def createSecretKey(self, size):
        return (''.join(map(lambda xx: (hex(ord(xx))[2:]), os.urandom(size))))[0:16]

    def views_capture(self, song_id, page=1, pages=1024):
        if pages > 1:   
            while page < pages:
                pages = self.view_capture(song_id, page)
                page = page + 1
        else:
            self.view_capture(song_id, 1)
        self.view_links(song_id)
    
    def view_capture(self, song_id, page=1):
        if page == 1:
            self.session.query(pysql.Comment163).filter(pysql.Comment163.song_id == song_id).delete()
            self.session.commit()
        data = {'params': self.createParams(page), 'encSecKey': self.__encSecKey}
        url = uapi.comment_url.format(str(song_id))
        try:
            req = requests.post(url, headers=self.__headers, data=data, timeout=10)
            for comment in req.json()['comments']:
                if comment['likedCount'] > 30:
                    txt = comment['content'].encode('utf-8')
                    author = comment['user']['nickname'].encode('utf-8')
                    liked = comment['likedCount']
                    self.session.add(pysql.Comment163(song_id=song_id, txt=txt, author=author, liked=liked))
                    self.session.flush()
            if page == 1:
                for comment in req.json()['hotComments']:
                    txt = comment['content'].encode('utf-8')
                    author = comment['user']['nickname'].encode('utf-8')
                    liked = comment['likedCount']
                    self.session.add(pysql.Comment163(song_id=song_id, txt=txt, author=author, liked=liked))
                    self.session.flush()
            cnt = int(req.json()['total'])
            self.session.query(pysql.Music163).filter(pysql.Music163.song_id == song_id).update({'over': 'Y', 'comment': cnt})
            self.session.commit()
            return cnt / 20
        except Exception as e:
            self.session.rollback()
            self.session.query(pysql.Music163).filter(pysql.Music163.song_id == song_id).update(
                {'over': 'Y', 'comment': -2})
            self.session.commit()
            pylog.log.error("解析歌曲评论的时候出现问题:{} 歌曲ID：{} 页码：{}".format(e, song_id, page))
            raise

    def view_links(self, song_id):
        url = "http://music.163.com/song?id=" + str(song_id)
        data = {'id': str(song_id)}
        headers = {'Cookie': 'MUSIC_U=e45797021db3403ab9fffb11c0f70a7994f71177b26efb5169b46948f2f9a60073d23a2665346106c9295f8f6dbb6c7731b299d667364ed3;'}
        try:
            req = requests.get(url, headers=headers, data=data, timeout=100)
            sup = BeautifulSoup(req.content, "html.parser")
            for link in sup.find_all('li', class_="f-cb"):
                html = link.find('a', 's-fc1')
                if html is not None:
                    title = html.get('title').encode('utf-8')
                    song_id = html.get('href')[9:]
                    author = link.find('div', 'f-thide s-fc4').find('span').get('title').encode('utf-8')
                    if pysql.single("music163","song_id",song_id) is True:
                        self.session.add(pysql.Music163(song_id=song_id, song_name=title, author=author))
                        self.session.flush()
            for link in sup.find_all('a', 'sname f-fs1 s-fc0'):
                play_link = link.get("href").replace("/playlist?id=", "")
                play_name = link.get("title").encode('utf-8')
                if pysql.single("playlist163", "link", play_link) is True:
                    self.session.add(pysql.Playlist163(title=play_name, link=play_link, cnt=-1, dsc="来源：热评"))
                    self.session.flush()
        except Exception as e:
            pylog.log.error("解析页面推荐时出现问题：{} 歌曲ID：{}".format(e, song_id))

    def auto_view(self, count=1):
        song = []
        try:
            if count < 10:
                msc = self.session.query(pysql.Music163).filter(pysql.Music163.over == "N").limit(count)
                for m in msc:
                    print("抓取热评 ID {} 歌曲 {}".format(m.song_id, pylog.Blue(m.song_name.encode('utf-8'))))
                    self.views_capture(m.song_id, 1, 1)
                    song.append({"name": m.song_name, "author": m.author, "song_id": m.song_id})
            else:
                for i in range(count / 10):
                    msc = self.session.query(pysql.Music163).filter(pysql.Music163.over == "N").limit(10)
                    for m in msc:
                        print("抓取热评 ID {} 歌曲 {}".format(m.song_id, pylog.Blue(m.song_name.encode('utf-8'))))
                        self.views_capture(m.song_id, 1, 1)
                        song.append({"name": m.song_name, "author": m.author, "song_id": m.song_id})
                msc = self.session.query(pysql.Music163).filter(pysql.Music163.over == "N").limit(count % 10)
                for m in msc:
                    print("抓取热评 ID {} 歌曲 {}".format(m.song_id, pylog.Blue(m.song_name.encode('utf-8'))))
                    self.views_capture(m.song_id, 1, 1)
                    song.append({"name": m.song_name, "author": m.author, "song_id": m.song_id})
        except Exception as e:
            self.session.rollback()
            pylog.log.error("自动抓取热评出现异常：{} 歌曲ID：{}".format(e, m.song_id))
            # raise
        return song

    def get_music(self, music_id):
        self.view_capture(int(music_id), 1)
        url = uapi.music_api.format(music_id, music_id)
        s = requests.session()
        s = BeautifulSoup(s.get(url, headers=self.__headers).content, "html.parser")
        music = json.loads(s.text)['songs']
        print("《" + music[0]['name'].encode('utf-8') + "》")
        author = []
        for a in music[0]['artists']:
            author.append(a['name'].encode('utf-8'))
        album = str(music[0]['album']['name'].encode('utf-8'))
        print("演唱：{}     专辑：{}".format("，".join(author), album))
        comments = self.session.query(pysql.Comment163).filter(pysql.Comment163.song_id == int(music_id))
        tb = AsciiTable([["序号", "作者", "评论", "点赞"]])
        max_width = tb.column_max_width(2) - tb.column_max_width(2) % 3
        cnt = 0
        try:
            for cmt in comments:
                cnt = cnt + 1
                au = cmt.author.encode("utf-8")
                txt = ""
                length = 0
                for u in cmt.txt:
                    txt = txt + u
                    if ord(u) < 128:
                        length = length + 3
                    else:
                        length = length + 1
                    if length == max_width:
                        txt = txt + "\n"
                        length = 0
                liked = str(cmt.liked)
                tb.table_data.append([str(cnt), str(au), str(txt), liked])
            print(tb.table)
        except UnicodeEncodeError:
            pylog.log.info("获取歌曲详情编码存在问题，转为非表格形式，歌曲ID：{}".format(music_id))
            for cmt in comments:
                print("评论： {}".format(cmt.txt.encode("utf-8")))
                print("作者： {}   点赞：  {}".format(cmt.author.encode("utf-8"), str(cmt.liked)))
                print("")
        except Exception as e:
            pylog.print_warn("获取歌曲时出现异常： {} 歌曲ID：{}".format(e, music_id))




#!/usr/bin/env python
# -*- coding: utf-8 -*

from setuptools import setup, find_packages, Command
import os
import imp


class CleanCommand(Command):
    """Custom clean command to tidy up the project root."""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


version = imp.load_source(
    'spider163.version', os.path.join('spider163', 'version.py')).VERSION

setup(
      version=version,
      name='spider163',
      author='ChengTian',
      entry_points={
        "console_scripts": ["spider163=spider163.bin.cli:main"]
      },
      url='https://github.com/Chengyumeng/spider163',
      author_email='792400644@qq.com',
      packages=find_packages(),
      include_package_data=True,
      license='MIT License',
      zip_safe=False,
      install_requires=[
            "beautifulsoup4==4.6.0",
            "bs4==0.0.1",
            "cement==2.10.2",
            "certifi==2017.7.27.1",
            "chardet==3.0.4",
            "idna==2.6",
            "MySQL-python==1.2.5",
            "Naked==0.1.31",
            "pprint==0.1",
            "progressbar==2.3",
            "pycrypto==2.6.1",
            "PyYAML==3.12",
            "requests==2.18.4",
            "shellescape==3.4.1",
            "SQLAlchemy==1.1.14",
            "SQLAlchemy-Utils==0.32.18",
            "terminaltables==3.1.0",
            "urllib3==1.22",
            "Logbook==1.1.0",
            "colorama==0.3.9",
            "Flask==0.12.2",

      ],
      cmdclass={
            'clean': CleanCommand,
      },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import random

from sqlalchemy import Column, Integer, String, TIMESTAMP, Index, extract
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import func
from sqlalchemy.dialects.mysql import MEDIUMTEXT

from spider163 import settings
from spider163.utils import pylog

Base = declarative_base()


class Playlist163(Base):
    __tablename__ = "playlist163"

    id = Column(Integer(), primary_key=True, autoincrement=True)
    title = Column(String(5000), server_default="System Title")
    link = Column(String(255), server_default="No Link")
    cnt = Column(Integer(), server_default="-1")
    dsc = Column(String(255), server_default="No Description")
    create_time = Column(TIMESTAMP, server_default=func.now())
    over = Column(String(255), server_default="N")
    over_link = Index("over_link", over, link)


class Music163(Base):
    __tablename__ = "music163"
    id = Column(Integer(), primary_key=True, autoincrement=True)
    song_id = Column(Integer())
    song_name = Column(String(5000), server_default="No Name")
    author = Column(String(3000), server_default="No Author")
    over = Column(String(255), server_default="N")
    has_lyric = Column(String(255), server_default="N")
    create_time = Column(TIMESTAMP, server_default=func.now())
    comment = Column(Integer(), server_default="-1")
    over_id = Index("over_id", over,id)
    key_author = Index("author", author)
    song_id_comment = Index("song_id_comment", song_id, comment)


class Comment163(Base):
    __tablename__ = "comment163"
    id = Column(Integer(), primary_key=True, autoincrement=True)
    song_id = Column(Integer())
    txt = Column(MEDIUMTEXT)
    author = Column(String(3000), server_default="No Author")
    liked = Column(Integer(), server_default="0")
    Index("liked_song_id", liked, song_id)
    Index("song_id_liked", song_id, liked)


class Lyric163(Base):
    __tablename__ = "lyric163"
    id = Column(Integer(), primary_key=True, autoincrement=True)
    song_id = Column(Integer())
    txt = Column(MEDIUMTEXT)
    key_song_id = Index("song_id", song_id)


def single(table, k, v):
    cnt = settings.engine.execute('select count(*) from ' + table + ' where ' + k + '=\'' + str(v) + '\'').fetchone()
    if cnt[0] == 0:
        return True
    else:
        return False


def stat_playlist():
    data = {}
    data["gdType"] = settings.Session.query(func.substring(Playlist163.dsc, 4, 2).label('type'), func.count('*').label('count')).group_by("type").all()
    data["gdOver"] = settings.Session.query(Playlist163.over.label('over'), func.count('*').label('count')).group_by("over").all()
    return data


def stat_music():
    data = {"author-comment-count": []}
    cd = settings.Session.query(Music163.author.label('author'), func.sum(Music163.comment).label('count')).group_by("author").order_by(func.sum(Music163.comment).label('count').label('count').desc()).limit(30).all()
    for m in cd:
        data["author-comment-count"].append([m[0], int(m[1])])
    data["music-comment-count"] = settings.Session.query(Music163.song_name, Music163.comment.label("count")).order_by(Music163.comment.label("count").desc()).limit(30).all()
    return data


def stat_data():
    data = {}
    data["countPlaylist"] = int(settings.engine.execute("select(select count(*) from playlist163 where over = 'Y')*100 / count(*) from playlist163").fetchone()[0]);
    data["countComment"] = int(settings.engine.execute("select(select count(*) from music163 where over = 'Y')*100 / count(*) from music163").fetchone()[0]);
    data["countLyric"] = int(settings.engine.execute("select(select count(*) from music163 where has_lyric = 'Y')*100 / count(*) from music163").fetchone()[0]);
    return data


def random_data():
    rng = settings.Session.query(func.min(Comment163.id), func.max(Comment163.id)).all()[0]
    data = []
    for i in range(12):
        v = random.uniform(rng[0], rng[1])
        d = settings.engine.execute("select txt,liked,a.author,song_name,a.song_id,b.author from comment163 a inner join music163 b on a.song_id= b.song_id where a.id>" +str(v) + " limit 1").fetchone()
        data.append({"txt": d[0],"like": d[1] ,"author": d[2],  "song" :{"name":d[3], "author": d[5], "id": d[4]}})
    return data


def initdb():
    try:
        Base.metadata.create_all(settings.engine)
    except Exception as e:
        pylog.print_warn("自动生成数据库表出现问题: {}".format(e))


def dropdb():
    try:
        Base.metadata.drop_all(settings.engine)
    except Exception as e:
        pylog.print_warn("自动删除数据库表出现问题: {}".format(e))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from spider163.utils import config
from spider163.utils import pylog


def is_correct_config():
    pylog.print_info("正在检查配置路径：{}".format(config.PATH))
    if not os.path.exists(config.PATH + "/spider163.conf"):
        print("  - 配置路径下 spider163.conf {}".format(pylog.red("不存在")))
    else:
        print("  - 配置路径下 spider163.conf {}".format(pylog.green("存在")))
    pylog.print_info("正在检查配置文件 {}/spider163.conf 内容是否完整".format(config.PATH))
    try:
        config.cf.get("core", "db")
        print("  - 配置文件中 db   选项      {}".format(pylog.green("存在")))
    except Exception:
        print("  - 配置文件中 db   选项      {}".format(pylog.red("不存在")))
    try:
        config.cf.get("core", "port")
        print("  - 配置文件中 port 选项      {}".format(pylog.green("存在")))
    except Exception:
        print("  - 配置文件中 port 选项      {}".format(pylog.red("不存在")))


def is_correct_db():
    db = config.format_db()
    pylog.print_info("正在检查配置的数据库格式和可用性")
    if db is None:
        print("  - 配置文件中 db 选项  {}".format(pylog.red("不正确")))
    else:
        print("  - 账号：{} 密码：{} IP：{} 数据库：{}".format(db["user"], db["password"], db["ip"], db["database"]))
        try:
            from sqlalchemy import create_engine
            create_engine(db["link"], echo=False).execute("show databases")
            print("数据库连接验证                {}".format(pylog.green("成功")))
        except Exception as e:
            pylog.print_err("数据库连接失败，上述配置信息有问题: {}".format(e))


def can_spider():
    print("抓取验证未完成")




# -*- coding: utf-8 -*-

from cement.core.foundation import CementApp
from cement.core.controller import CementBaseController, expose
from cement.core.exc import CaughtSignal
from colorama import Fore
from colorama import init

from spider163.utils import pysql
from spider163.spider import playlist
from spider163.spider import music
from spider163.spider import comment
from spider163.spider import lyric
from spider163.spider import search
from spider163 import version
from spider163.www import web
from spider163.utils import config
from spider163.utils import pylog
from spider163.utils import healthz

BANNER = """
Spider163 Application v%s
Copyright (c) 2017 Cheng Tian Enterprises
Welcome to Follow My 【微信公众账号】"程天写代码"
""" % version.VERSION

init(autoreset=True)


class VersionController(CementBaseController):
    class Meta:
        label = 'base'
        description = 'Spider163是Github上最流行的网易云音乐爬虫系统'
        arguments = [
            (['-v', '--version'], dict(action='version', version=BANNER)),
            ]

    @expose(help="运行前健康检查")
    def healthz(self):
        healthz.is_correct_config()
        healthz.is_correct_db()
        healthz.can_spider()


class DatabaseController(CementBaseController):
    class Meta:
        label = "database"
        description = "数据库相关操作"
        arguments = []

    @expose(help="自动生成数据库相关依赖")
    def initdb(self):
        print("正在生成全部数据库表结构……")
        pysql.initdb()

    @expose(help="重置数据库配置")
    def resetdb(self):
        print("正在删除全部已下载数据……")
        pysql.dropdb()
        pysql.initdb()


class SpiderController(CementBaseController):
    class Meta:
        label = "spider"
        description = "爬虫-蜘蛛等相关操作"
        arguments = [
            (['-p', '--page'],
             dict(help="抓取的页码")),
            (['-c', '--count'],
             dict(help="")),
            (['-s', '--song'],
             dict(help="歌曲ID")),
            (['--classify'],
             dict(help="歌曲风格")),
        ]

    @expose(help="获取全部歌曲风格列表(作为抓取歌单的参照)")
    def classify(self):
        playlist.Playlist().get_classify()

    @expose(help="根据推荐歌单抓取网易云音乐歌单数据(-p --page | --classify)")
    def playlist(self):
        pg = self.app.pargs.page
        cf = "全部"
        pl = playlist.Playlist()
        if self.app.pargs.classify is not None:
            cf = self.app.pargs.classify
        if pg is not None:
            print(Fore.GREEN + '正在抓取 曲风为 {} 的第 {} 页歌单……'.format(cf, pg))
            pl.view_capture(int(pg), cf)
        else:
            for i in range(36):
                print(Fore.GREEN + '正在抓取 曲风为 {} 的第 {} 页歌单……'.format(cf ,i + 1))
                pl.view_capture(i + 1, cf)

    @expose(help="通过歌单抓取网易云音乐歌曲，单次抓取歌单10个(-c --count)")
    def music(self):
        msc = music.Music()
        if self.app.pargs.count is None:
            msc.views_capture()
            return
        cnt = int(self.app.pargs.count)
        if cnt <= 0:
            print(Fore.RED + "不合法的--count -c 变量（ > 0 ）")
        else:
            for i in range(cnt):
                print(Fore.GREEN + '正在执行第 {} 批抓取计划，本次抓取歌单歌曲 10 个\r\n'.format(i + 1))
                msc.views_capture()

    @expose(help="通过音乐列表抓取网易云音乐热评，单次抓取音乐1首(-c --count),也可以指定歌曲ID(-s --song)")
    def comment(self):
        cmt = comment.Comment()
        if self.app.pargs.song is not None:
            print(Fore.BLUE + '正在执行抓取歌曲 {} 热门评论计划'.format(self.app.pargs.song))
            cmt.view_capture(int(self.app.pargs.song), 1)
            print(Fore.GREEN + '抓取完成\r\n')
            return
        if self.app.pargs.count is not None:
            print(Fore.GREEN + '正在执行批量抓取热门评论计划，本次计划抓取歌曲 {} 首\r\n'.format(self.app.pargs.count))
            cmt.auto_view(int(self.app.pargs.count))
        else:
            cmt.auto_view(1)

    @expose(help="通过音乐列表抓取网易云音乐歌词,可以指定抓取歌曲数量（-c --count），也可以指定歌曲ID（-s --song）")
    def lyric(self):
        lrc = lyric.Lyric()
        if self.app.pargs.song is not None:
            print(Fore.BLUE + '正在执行抓取歌曲 {} 歌词的计划'.format(self.app.pargs.song))
            lrc.view_lyric(self.app.pargs.song)
            print(Fore.GREEN + '抓取完成\r\n')
        elif self.app.pargs.count is not None:
            print(Fore.GREEN + '正在执行批量抓取歌词计划，本次计划抓取歌曲 {} 首\r\n'.format(self.app.pargs.count))
            lrc.view_lyrics(int(self.app.pargs.count))
        else:
            print("您至少指定--song或者--count一个参数")


class QueryController(CementBaseController):
    class Meta:
        label = "query"
        stacked_on = 'base'
        description = "爬虫-蜘蛛等相关操作"
        arguments = [
            (['--playlist'],
             dict(help="")),
            (['-q', '--query'],
             dict(help="")),
        ]

    @expose(help="通过歌单ID和歌曲ID获取歌单、歌曲相关信息（--song --playlist）")
    def get(self):
        if self.app.pargs.song is not None:
            comment.Comment().get_music(self.app.pargs.song)
            lyric.Lyric().get_lyric(self.app.pargs.song)
        if self.app.pargs.playlist is not None:
            music.Music().get_playlist(self.app.pargs.playlist)

    @expose(help="搜索功能(-q --query)")
    def search(self):
        if self.app.pargs.query is not None:
            search.searchSong(self.app.pargs.query)
            search.searchAlbum(self.app.pargs.query)
            search.searchSinger(self.app.pargs.query)
            search.searchPlaylist(self.app.pargs.query)


class WebController(CementBaseController):
    class Meta:
        label = "web"
        stacked_on = 'base'
        description = "网络平台"
        arguments = [
        ]

    @expose(help="Spider163管理Web平台")
    def webserver(self):
        try:
            webport = config.get_port()
            web.app.run(host="0.0.0.0", port=webport, debug=True)
        except Exception as e:
            print("{} 退出web服务：{}".format(Fore.RED, e))


class App(CementApp):
    class Meta:
        label = "Spider163"
        base_controller = "base"
        handlers = [VersionController, DatabaseController, SpiderController, QueryController, WebController]


def main():
    with App() as app:
        try:
            app.run()
        except CaughtSignal as e:
            pylog.print_warn("控制台异常：{}".format(e))
        except Exception as e:
            pylog.print_err("执行抓取任务遭遇配置异常： {}".format(e))




